@@ -1,6 +1,6 @@
-// Copyright 2008-present Contributors to the OpenImageIO project.
-// SPDX-License-Identifier: BSD-3-Clause
-// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
+// Copyright Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: Apache-2.0
+// https://github.com/AcademySoftwareFoundation/OpenImageIO
 
 
 /// \file
@@ -16,20 +16,66 @@
 
 #include <OpenImageIO/attrdelegate.h>
 #include <OpenImageIO/export.h>
+#include <OpenImageIO/memory.h>
+#include <OpenImageIO/strongparam.h>
 #include <OpenImageIO/typedesc.h>
 #include <OpenImageIO/ustring.h>
 
 
 OIIO_NAMESPACE_BEGIN
 
-/// ParamValue holds a parameter and a pointer to its value(s)
+/// ParamValue holds a named parameter and typed data. Usually, it owns the
+/// data (holding it in the struct itself if small enough, dynamically
+/// allocated for larger things), but it can also refer to non-owned data.
 ///
-/// Nomenclature: if you have an array of 4 colors for each of 15 points...
-///  - There are 15 VALUES
-///  - Each value has an array of 4 ELEMENTS, each of which is a color
+/// The data is usually a single value of any type described by TypeDesc
+/// (including arrays). It may also hold more than one value of the type --
+/// this is usually only used in a geometric context, such as storing a value
+/// for each vertex in a mesh. Please note the subtle distinction between the
+/// value type itself being an array, versus having multiple values as a
+/// parameter, versus the type of the value having multiple components (such
+/// as a point or color). Any combination of these may be present.
+///
+/// To clarify, if you have an array of 4 colors for each of 15 mesh
+/// vertices, that means:
+///  - There are 15 VALUES (one for each vertex)
+///  - Each value has an array of 4 ELEMENTS
+///  - Each element is a color
 ///  - A color has 3 COMPONENTS (R, G, B)
+/// Thus, it would be constructed as
+/// `ParamValue("mycolor", TypeDesc(TypeDesc::COLOR, 4), 15, ptr_to_data)`
 ///
-class OIIO_API ParamValue {
+/// The main constructor is `ParamValue(name, type, nvalues, dataptr)`. It can
+/// be confusing at first to remember that the data argument is a pointer to
+/// the first values to copy, not the values themselves, even if the values
+/// are themselves pointers, and even if the number of values is 1. In other
+/// words, it's behaving as if you're always pointing it to an array even if
+/// the "array" has only one element. This is extra confusing for strings,
+/// because the strings themselves are `char*` (or ustring), so the pointer
+/// you need to pass is `char**`. For this reason, there are also convenience
+/// constructors for simple types such as a single int, float, or string.
+///
+/// So here are some examples:
+///
+///     // Single int:
+///     int my_int = 42;
+///     ParamValue A("foo", TypeDesc::INT, 1, &my_int);
+///     // Three int values (say, one per vertex of a triangle):
+///     int my_int_array[3] = { 1, 2, 3 };
+///     ParamValue B("foo", TypeDesc::INT, 1, &my_int_array);
+///     // A single value which is an array of 3 ints:
+///     ParamValue C("foo", TypeDesc(TypeDesc::INT, 3), 1, &my_int_array);
+///     // A string -- note the trick about treating it as an array:
+///     const char* my_string = "hello";
+///     ParamValue D("foo", TypeDesc::STRING, 1, &my_string);
+///
+///     // The most common cases also have simple "duck-typed" convenience
+///     // constructors:
+///     ParamValue A("foo", 42);           // single int
+///     ParamValue B("foo", 42.0f);        // single float
+///     ParamValue C("foo", "forty two");  // single string
+
+class OIIO_UTIL_API ParamValue {
 public:
     /// Interpolation types
     ///
@@ -40,27 +86,33 @@ public:
         INTERP_VERTEX   = 3   ///< Interpolated like vertices
     };
 
+    OIIO_STRONG_PARAM_TYPE(Copy, bool);
+    OIIO_STRONG_PARAM_TYPE(FromUstring, bool);
+
     ParamValue() noexcept { m_data.ptr = nullptr; }
+
     ParamValue(const ustring& _name, TypeDesc _type, int _nvalues,
-               const void* _value, bool _copy = true) noexcept
+               const void* _value, Copy _copy = Copy(true)) noexcept
     {
         init_noclear(_name, _type, _nvalues, _value, _copy);
     }
     ParamValue(const ustring& _name, TypeDesc _type, int _nvalues,
-               Interp _interp, const void* _value, bool _copy = true) noexcept
+               Interp _interp, const void* _value,
+               Copy _copy = Copy(true)) noexcept
     {
         init_noclear(_name, _type, _nvalues, _interp, _value, _copy);
     }
     ParamValue(string_view _name, TypeDesc _type, int _nvalues,
-               const void* _value, bool _copy = true) noexcept
+               const void* _value, Copy _copy = Copy(true)) noexcept
     {
         init_noclear(ustring(_name), _type, _nvalues, _value, _copy);
     }
     ParamValue(string_view _name, TypeDesc _type, int _nvalues, Interp _interp,
-               const void* _value, bool _copy = true) noexcept
+               const void* _value, Copy _copy = Copy(true)) noexcept
     {
         init_noclear(ustring(_name), _type, _nvalues, _interp, _value, _copy);
     }
+
     ParamValue(string_view _name, int value) noexcept
     {
         init_noclear(ustring(_name), TypeDesc::INT, 1, &value);
@@ -71,12 +123,17 @@ public:
     }
     ParamValue(string_view _name, ustring value) noexcept
     {
-        init_noclear(ustring(_name), TypeDesc::STRING, 1, &value);
+        init_noclear(ustring(_name), TypeDesc::STRING, 1, &value, Copy(true),
+                     FromUstring(true));
     }
     ParamValue(string_view _name, string_view value) noexcept
+        : ParamValue(_name, ustring(value))
     {
-        ustring u(value);
-        init_noclear(ustring(_name), TypeDesc::STRING, 1, &u);
+    }
+    ParamValue(string_view _name, ustringhash value) noexcept
+    {
+        init_noclear(ustring(_name), TypeDesc::USTRINGHASH, 1, &value,
+                     Copy(true));
     }
 
     // Set from string -- parse
@@ -86,19 +143,19 @@ public:
     ParamValue(const ParamValue& p) noexcept
     {
         init_noclear(p.name(), p.type(), p.nvalues(), p.interp(), p.data(),
-                     true);
+                     Copy(true), FromUstring(true));
     }
-    ParamValue(const ParamValue& p, bool _copy) noexcept
+    ParamValue(const ParamValue& p, Copy _copy) noexcept
     {
         init_noclear(p.name(), p.type(), p.nvalues(), p.interp(), p.data(),
-                     _copy);
+                     _copy, FromUstring(true));
     }
 
     // Rvalue (move) constructor
     ParamValue(ParamValue&& p) noexcept
     {
         init_noclear(p.name(), p.type(), p.nvalues(), p.interp(), p.data(),
-                     false);
+                     Copy(false), FromUstring(true));
         m_copy       = p.m_copy;
         m_nonlocal   = p.m_nonlocal;
         p.m_data.ptr = nullptr;  // make sure the old one won't free
@@ -107,45 +164,30 @@ public:
     ~ParamValue() noexcept { clear_value(); }
 
     void init(ustring _name, TypeDesc _type, int _nvalues, Interp _interp,
-              const void* _value, bool _copy = true) noexcept
+              const void* _value, Copy _copy) noexcept
     {
         clear_value();
         init_noclear(_name, _type, _nvalues, _interp, _value, _copy);
     }
     void init(ustring _name, TypeDesc _type, int _nvalues, const void* _value,
-              bool _copy = true) noexcept
+              Copy _copy = Copy(true)) noexcept
     {
         init(_name, _type, _nvalues, INTERP_CONSTANT, _value, _copy);
     }
     void init(string_view _name, TypeDesc _type, int _nvalues,
-              const void* _value, bool _copy = true) noexcept
+              const void* _value, Copy _copy = Copy(true)) noexcept
     {
         init(ustring(_name), _type, _nvalues, _value, _copy);
     }
     void init(string_view _name, TypeDesc _type, int _nvalues, Interp _interp,
-              const void* _value, bool _copy = true) noexcept
+              const void* _value, Copy _copy = Copy(true)) noexcept
     {
         init(ustring(_name), _type, _nvalues, _interp, _value, _copy);
     }
 
     // Assignment
-    const ParamValue& operator=(const ParamValue& p) noexcept
-    {
-        if (this != &p)
-            init(p.name(), p.type(), p.nvalues(), p.interp(), p.data(),
-                 p.m_copy);
-        return *this;
-    }
-    const ParamValue& operator=(ParamValue&& p) noexcept
-    {
-        if (this != &p) {
-            init(p.name(), p.type(), p.nvalues(), p.interp(), p.data(), false);
-            m_copy       = p.m_copy;
-            m_nonlocal   = p.m_nonlocal;
-            p.m_data.ptr = nullptr;
-        }
-        return *this;
-    }
+    const ParamValue& operator=(const ParamValue& p) noexcept;
+    const ParamValue& operator=(ParamValue&& p) noexcept;
 
     // FIXME -- some time in the future (after more cleanup), we should make
     // name() return a string_view, and use uname() for the rare time when
@@ -173,24 +215,46 @@ public:
         b        = std::move(tmp);
     }
 
+    /// Return a `cspan<T>` pointing to the bounded data. If the type `T` is
+    /// not the actual underlying base type, return an empty span.
+    template<typename T> cspan<T> as_cspan() const noexcept
+    {
+        return BaseTypeFromC<T>::value == m_type.basetype
+                   ? make_cspan(reinterpret_cast<const T*>(data()),
+                                size_t(m_nvalues * m_type.basevalues()))
+                   : cspan<T>();
+    }
+
+    /// Return a `span<T>` pointing to the bounded data. If the type `T` is
+    /// not the actual underlying base type, return an empty span.
+    template<typename T> span<T> as_span() noexcept
+    {
+        return BaseTypeFromC<T>::value == m_type.basetype
+                   ? make_span(reinterpret_cast<T*>(const_cast<void*>(data())),
+                               size_t(m_nvalues * m_type.basevalues()))
+                   : span<T>();
+    }
+
     // Use with extreme caution! This is just doing a cast. You'd better
     // be really sure you are asking for the right type. Note that for
     // "string" data, you can get<ustring> or get<char*>, but it's not
     // a std::string.
     template<typename T> const T& get(int i = 0) const noexcept
     {
+        OIIO_DASSERT(i >= 0 && i < int(m_nvalues * m_type.basevalues())
+                     && "OIIO::ParamValue::get() range check");
         return (reinterpret_cast<const T*>(data()))[i];
     }
 
-    /// Retrive an integer, with converstions from a wide variety of type
-    /// cases, including unsigned, short, byte. Not float. It will retrive
+    /// Retrieve an integer, with conversions from a wide variety of type
+    /// cases, including unsigned, short, byte. Not float. It will retrieve
     /// from a string, but only if the string is entirely a valid int
     /// format. Unconvertible types return the default value.
     int get_int(int defaultval = 0) const;
     int get_int_indexed(int index, int defaultval = 0) const;
 
-    /// Retrive a float, with converstions from a wide variety of type
-    /// cases, including integers. It will retrive from a string, but only
+    /// Retrieve a float, with conversions from a wide variety of type
+    /// cases, including integers. It will retrieve from a string, but only
     /// if the string is entirely a valid float format. Unconvertible types
     /// return the default value.
     float get_float(float defaultval = 0) const;
@@ -200,12 +264,16 @@ public:
     /// elements is also passed. In the case of a single string, just the
     /// string directly is returned. But for an array of strings, the array
     /// is returned as one string that's a comma-separated list of double-
-    /// quoted, escaped strings.
+    /// quoted, escaped strings. For an array or aggregate, at most `maxsize`
+    /// elements are returned (if `maxsize` is 0, all elements are returned,
+    /// no matter how large it is).
     std::string get_string(int maxsize = 64) const;
     std::string get_string_indexed(int index) const;
     /// Convert any type to a ustring value. An optional maximum number of
-    /// elements is also passed. Same behavior as get_string, but returning
-    /// a ustring.
+    /// elements is also passed. Same behavior as get_string, but returning a
+    /// ustring. For an array or aggregate, at most `maxsize` elements are
+    /// returned (if `maxsize` is 0, all elements are returned, no matter how
+    /// large it is).
     ustring get_ustring(int maxsize = 64) const;
     ustring get_ustring_indexed(int index) const;
 
@@ -222,19 +290,48 @@ private:
     bool m_nonlocal        = false;
 
     void init_noclear(ustring _name, TypeDesc _type, int _nvalues,
-                      const void* _value, bool _copy = true) noexcept;
+                      const void* _value, Copy _copy = Copy(true),
+                      FromUstring _from_ustring = FromUstring(false)) noexcept;
     void init_noclear(ustring _name, TypeDesc _type, int _nvalues,
                       Interp _interp, const void* _value,
-                      bool _copy = true) noexcept;
+                      Copy _copy                = Copy(true),
+                      FromUstring _from_ustring = FromUstring(false)) noexcept;
     void clear_value() noexcept;
+
+    /// declare a friend heapsize definition
+    template<typename T> friend size_t pvt::heapsize(const T&);
 };
 
+/// heapsize specialization for `ParamValue`
+template<>
+OIIO_API size_t
+pvt::heapsize<ParamValue>(const ParamValue&);
+
+/// Factory for a ParamValue that holds a single value of any type supported
+/// by a corresponding ParamValue constructor (such as int, float, string).
+template<typename T>
+static ParamValue
+make_pv(string_view name, const T& val)
+{
+    return ParamValue(name, val);
+}
+
+/// Factory for a ParamValue from a pointer. Passing `char*` or `const char*`
+/// will be interpreted as a C string (TypeString), but all other pointer
+/// types will just get stored as an opaque pointer (TypePointer).
+template<typename T>
+static ParamValue
+make_pv(string_view name, T* val)
+{
+    return ParamValue(name, BaseTypeFromC<T*>::value, 1, &val);
+}
+
 
 
 /// A list of ParamValue entries, that can be iterated over or searched.
 /// It's really just a std::vector<ParamValue>, but with a few more handy
 /// methods.
-class OIIO_API ParamValueList : public std::vector<ParamValue> {
+class OIIO_UTIL_API ParamValueList : public std::vector<ParamValue> {
 public:
     ParamValueList() {}
 
@@ -274,17 +371,17 @@ public:
         return f != cend() ? &(*f) : nullptr;
     }
 
-    /// Case insensitive search for an integer, with default if not found.
-    /// Automatically will return an int even if the data is really
-    /// unsigned, short, or byte, but not float. It will retrive from a
-    /// string, but only if the string is entirely a valid int format.
+    /// Search for an integer, with default if not found. Automatically will
+    /// return an int even if the data is really unsigned, short, or byte, but
+    /// not float. It will retrieve from a string, but only if the string is
+    /// entirely a valid int format.
     int get_int(string_view name, int defaultval = 0,
                 bool casesensitive = false, bool convert = true) const;
 
-    /// Case insensitive search for a float, with default if not found.
-    /// Automatically will return a float even if the data is really double
-    /// or half. It will retrive from a string, but only if the string is
-    /// entirely a valid float format.
+    /// Search for a float, with default if not found. Automatically will
+    /// return a float even if the data is really double or half. It will
+    /// retrieve from a string, but only if the string is entirely a valid
+    /// float format.
     float get_float(string_view name, float defaultval = 0,
                     bool casesensitive = false, bool convert = true) const;
 
@@ -350,6 +447,12 @@ public:
         attribute(name, TypeString, 1, &v);
     }
 
+    void attribute(string_view name, ustring value)
+    {
+        if (!name.empty())
+            add_or_replace(ParamValue(name, value));
+    }
+
     /// Search list for named item, return its type or TypeUnknown if not
     /// found.
     TypeDesc getattributetype(string_view name,
@@ -442,4 +545,149 @@ public:
 
 
 
+/// A span of const ParamValue entries, that can be iterated over or searched.
+/// It's really just a cspan<ParamValue>, but with a few more handy methods.
+/// This is a convenient way to pass the contents of a ParamValueList (or its
+/// equivalent of any consecutive subarray of PV's) without any copies.
+class OIIO_UTIL_API ParamValueSpan : public cspan<ParamValue> {
+public:
+    // Note: inherits from cspan:
+    //   - size()
+    //   - data()
+    //   - operator[int]
+    //   - begin(), end(), cbegin(), cend()
+
+    ParamValueSpan() = default;
+
+    ParamValueSpan(cspan<ParamValue> p)
+        : cspan<ParamValue>(p)
+    {
+    }
+
+    // Trivially make a ParamValueSpan from a ParamValueList
+    ParamValueSpan(const ParamValueList& p)
+        : cspan<ParamValue>(p)
+    {
+    }
+
+    /// Construct a span from an initializer_list.
+    constexpr ParamValueSpan(std::initializer_list<ParamValue> il)
+        : cspan<ParamValue>(il.begin(), il.size())
+    {
+    }
+
+    /// Construct from a fixed-length C array.  Template magic automatically
+    /// finds the length from the declared type of the array.
+    template<size_t N>
+    constexpr ParamValueSpan(const ParamValue (&data)[N])
+        : cspan<ParamValue>(data, N)
+    {
+    }
+
+    constexpr reference operator[](size_type idx) const
+    {
+        return cspan<ParamValue>::operator[](idx);
+    }
+
+    const_iterator find(string_view name, TypeDesc type = TypeUnknown,
+                        bool casesensitive = false) const;
+    const_iterator find(ustring name, TypeDesc type = TypeUnknown,
+                        bool casesensitive = false) const;
+
+    /// Search for an integer, with default if not found. Automatically will
+    /// return an int even if the data is really unsigned, short, or byte, but
+    /// not float. It will retrieve from a string, but only if the string is
+    /// entirely a valid int format.
+    int get_int(string_view name, int defaultval = 0,
+                bool casesensitive = false, bool convert = true) const;
+    int get_int(ustring name, int defaultval = 0, bool casesensitive = false,
+                bool convert = true) const;
+
+    /// Search for a float, with default if not found. Automatically will
+    /// return a float even if the data is really double or half. It will
+    /// retrieve from a string, but only if the string is entirely a valid
+    /// float format.
+    float get_float(string_view name, float defaultval = 0,
+                    bool casesensitive = false, bool convert = true) const;
+    float get_float(ustring name, float defaultval = 0,
+                    bool casesensitive = false, bool convert = true) const;
+
+    /// Simple way to get a string attribute, with default provided.
+    /// If the value is another type, it will be turned into a string.
+    string_view get_string(string_view name,
+                           string_view defaultval = string_view(),
+                           bool casesensitive     = false,
+                           bool convert           = true) const;
+    string_view get_string(ustring name, string_view defaultval = string_view(),
+                           bool casesensitive = false,
+                           bool convert       = true) const;
+    ustring get_ustring(string_view name,
+                        string_view defaultval = string_view(),
+                        bool casesensitive = false, bool convert = true) const;
+    ustring get_ustring(ustring name, string_view defaultval = string_view(),
+                        bool casesensitive = false, bool convert = true) const;
+
+    /// Search for the attribute and return its "truth-like" value: false if
+    /// it exists but is empty, or is a numeric value equal to 0, or a string
+    /// value that is "0", "no", "off", or "false". Otherwise, any non-empty
+    /// value returns true.
+    bool get_bool(string_view name, bool defaultval = false,
+                  bool casesensitive = false) const;
+    bool get_bool(ustring name, bool defaultval = false,
+                  bool casesensitive = false) const;
+
+    /// Does the span contain the named attribute?
+    bool contains(string_view name, TypeDesc type = TypeUnknown,
+                  bool casesensitive = false) const
+    {
+        return (find(name, type, casesensitive) != end());
+    }
+    bool contains(ustring name, TypeDesc type = TypeUnknown,
+                  bool casesensitive = false) const
+    {
+        return (find(name, type, casesensitive) != end());
+    }
+
+    /// Search list for named item, return its type or TypeUnknown if not
+    /// found.
+    TypeDesc getattributetype(string_view name,
+                              bool casesensitive = false) const
+    {
+        auto p = find(name, TypeUnknown, casesensitive);
+        return p != cend() ? p->type() : TypeUnknown;
+    }
+
+    /// Retrieve from list: If found and its data type is reasonably convertible
+    /// to `type`, copy/convert the value into val[...] and return true.
+    /// Otherwise, return false and don't modify what val points to.
+    bool getattribute(string_view name, TypeDesc type, void* value,
+                      bool casesensitive = false) const;
+    /// Shortcut for retrieving a single string via getattribute.
+    bool getattribute(string_view name, std::string& value,
+                      bool casesensitive = false) const;
+
+    /// Retrieve from list: If found its data type is reasonably convertible
+    /// to `type`, copy/convert the value into val[...] and return true.
+    /// Otherwise, return false and don't modify what val points to.
+    bool getattribute_indexed(string_view name, int index, TypeDesc type,
+                              void* value, bool casesensitive = false) const;
+    /// Shortcut for retrieving a single string via getattribute.
+    bool getattribute_indexed(string_view name, int index, std::string& value,
+                              bool casesensitive = false) const;
+
+    // Inherits operator[int] from span
+
+    /// Array indexing by string will create a "Delegate" that enables a
+    /// convenient shorthand for retrieving a value:
+    ///
+    ///     int i = list["foo"].get<int>();
+    ///     std::string s = list["baz"].get<std::string>();
+    ///
+    AttrDelegate<const ParamValueSpan> operator[](string_view name) const
+    {
+        return { this, name };
+    }
+};
+
+
 OIIO_NAMESPACE_END
