@@ -94,10 +94,10 @@ public:
     using string = std::basic_string<CharT, Traits>;
     static const size_type npos = ~size_type(0);
 
-    /// Default ctr
+    /// Default constructor.
     constexpr basic_string_view() noexcept : m_chars(nullptr), m_len(0) { }
 
-    /// Copy ctr
+    /// Copy constructor.
     constexpr basic_string_view(const basic_string_view& copy)
         : m_chars(copy.data()), m_len(copy.size()) { }
 
@@ -107,7 +107,7 @@ public:
 
     /// Construct from char*, use strlen to determine length.
     constexpr basic_string_view(const CharT* chars) noexcept
-        : m_chars(chars), m_len(chars ? cestrlen(chars) : 0) { }
+        : m_chars(chars), m_len(chars ? Traits::length(chars) : 0) { }
 
     /// Construct from std::string. Remember that a string_view doesn't have
     /// its own copy of the characters, so don't use the `string_view` after
@@ -139,28 +139,14 @@ public:
         // N.B. std::string ctr from chars+len is constexpr in C++20.
     }
 
-    /// Explicitly request a 0-terminated string. USUALLY, this turns out to
-    /// be just data(), with no significant added expense (because most uses
-    /// of string_view are simple wrappers of C strings, C++ std::string, or
-    /// ustring -- all of which are 0-terminated). But in the more rare case
-    /// that the string_view represents a non-0-terminated substring, it
-    /// will force an allocation and copy underneath.
-    ///
-    /// Caveats:
-    /// 1. This is NOT going to be part of the C++17 std::string_view, so
-    ///    it's probably best to avoid this method if you want to have 100%
-    ///    drop-in compatibility with std::string_view.
-    /// 2. It is NOT SAFE to use c_str() on a string_view whose last char
-    ///    is the end of an allocation -- because that next char may only
-    ///    *coincidentally* be a '\0', which will cause c_str() to return
-    ///    the string start (thinking it's a valid C string, so why not just
-    ///    return its address?), if there's any chance that the subsequent
-    ///    char could change from 0 to non-zero during the use of the result
-    ///    of c_str(), and thus break the assumption that it's a valid C str.
+    /// DEPRECATED(3.0) -- If you must use this at all, use the freestanding
+    /// OIIO::c_str(OIIO::string_view). We want to get this out of the
+    /// OIIO::string_view template to preserve symmetry with std::string_view.
+    OIIO_DEPRECATED("Unsafe, nonstandard. Use standalone OIIO::c_str(string_view) if you must. (3.0)")
     const CharT* c_str() const;
 
-    // Assignment
-    OIIO_CONSTEXPR14 basic_string_view& operator=(const basic_string_view& copy) noexcept = default;
+    /// Assignment
+    constexpr basic_string_view& operator=(const basic_string_view& copy) noexcept = default;
 
     /// Convert a string_view to a `std::string`.
     operator std::basic_string<CharT, Traits>() const {
@@ -168,7 +154,7 @@ public:
     }
 
 #if defined(OIIO_STD_STRING_VIEW_AVAILABLE) || defined(OIIO_DOXYGEN)
-    // Convert an OIIO::string_view to a std::string_view.
+    /// Convert an OIIO::string_view to a std::string_view.
     constexpr operator std::basic_string_view<CharT, Traits>() const
     {
         return { data(), size() };
@@ -176,7 +162,7 @@ public:
 #endif
 
 #ifdef OIIO_EXPERIMENTAL_STRING_VIEW_AVAILABLE
-    // Convert an OIIO::string_view to a std::experimental::string_view.
+    /// Convert an OIIO::string_view to a std::experimental::string_view.
     constexpr operator std::experimental::basic_string_view<CharT, Traits>() const noexcept
     {
         return { data(), size() };
@@ -190,55 +176,78 @@ public:
 #endif
 
     // iterators
-    constexpr iterator begin() const noexcept { return m_chars; }
-    constexpr iterator end() const noexcept { return m_chars + m_len; }
-    constexpr const_iterator cbegin() const noexcept { return m_chars; }
-    constexpr const_iterator cend() const noexcept { return m_chars + m_len; }
-    OIIO_CONSTEXPR17 reverse_iterator rbegin() const noexcept { return reverse_iterator (end()); }
-    OIIO_CONSTEXPR17 reverse_iterator rend() const noexcept { return reverse_iterator (begin()); }
-    OIIO_CONSTEXPR17 const_reverse_iterator crbegin() const noexcept { return const_reverse_iterator (cend()); }
-    OIIO_CONSTEXPR17 const_reverse_iterator crend() const noexcept { return const_reverse_iterator (cbegin()); }
 
-    // capacity
+    /// Iterator pointing to the first char.
+    constexpr iterator begin() const noexcept { return m_chars; }
+    /// Iterator pointing to one past the last char.
+    constexpr iterator end() const noexcept { return m_chars + m_len; }
+    /// Const iterator pointing to the first char.
+    constexpr const_iterator cbegin() const noexcept { return m_chars; }
+    /// Const iterator pointing to one past the last char.
+    constexpr const_iterator cend() const noexcept { return m_chars + m_len; }
+    /// Reverse iterator pointing to the last char.
+    constexpr reverse_iterator rbegin() const noexcept { return reverse_iterator (end()); }
+    /// Reverse iterator pointing to one before the first char.
+    constexpr reverse_iterator rend() const noexcept { return reverse_iterator (begin()); }
+    /// Const reverse iterator pointing to the last char.
+    constexpr const_reverse_iterator crbegin() const noexcept { return const_reverse_iterator (cend()); }
+    /// Const reverse iterator pointing to one before the first char.
+    constexpr const_reverse_iterator crend() const noexcept { return const_reverse_iterator (cbegin()); }
+
+
+    /// Return the number of elements in the view.
     constexpr size_type size() const noexcept { return m_len; }
+    /// Return the number of elements in the view.
     constexpr size_type length() const noexcept { return m_len; }
     constexpr size_type max_size() const noexcept {
         return std::numeric_limits<size_type>::max();
     }
-    /// Is the basic_string_view empty, containing no characters?
+    /// Is the view empty, containing no characters?
     constexpr bool empty() const noexcept { return m_len == 0; }
 
-    /// Element access of an individual character (beware: no bounds
-    /// checking!).
+    /// Element access of an individual character. For debug build, does
+    /// bounds check with assertion. For optimized builds, there is no bounds
+    /// check.  Note: this is different from C++ std::span, which never bounds
+    /// checks `operator[]`.
     constexpr const_reference operator[](size_type pos) const { return m_chars[pos]; }
     /// Element access with bounds checking and exception if out of bounds.
-    OIIO_CONSTEXPR17 const_reference at(size_t pos) const
+    constexpr const_reference at(size_t pos) const
     {
         if (pos >= m_len)
             throw(std::out_of_range("OpenImageIO::string_view::at"));
         return m_chars[pos];
     }
+    /// The first character of the view.
     constexpr const_reference front() const { return m_chars[0]; }
+    /// The last character of the view.
     constexpr const_reference back() const { return m_chars[m_len - 1]; }
+    /// Return the underlying data pointer to the first character.
     constexpr const_pointer data() const noexcept { return m_chars; }
 
     // modifiers
-    OIIO_CONSTEXPR14 void clear() noexcept { init(nullptr, 0); }
-    OIIO_CONSTEXPR14 void remove_prefix(size_type n) noexcept
+
+    /// Reset the view to an empty string.
+    constexpr void clear() noexcept { init(nullptr, 0); }
+    /// Remove the first n characters from the view.
+    constexpr void remove_prefix(size_type n) noexcept
     {
         if (n > m_len)
             n = m_len;
         m_chars += n;
         m_len -= n;
     }
-    OIIO_CONSTEXPR14 void remove_suffix(size_type n) noexcept
+    /// Remove the last n characters from the view.
+    constexpr void remove_suffix(size_type n) noexcept
     {
         if (n > m_len)
             n = m_len;
         m_len -= n;
     }
 
-    OIIO_CONSTEXPR14 basic_string_view substr(size_type pos, size_type n = npos) const noexcept
+    /// Return a new string_view that is a substring of this one, starting at
+    /// position pos and of length n. If n is npos, it will be the rest of the
+    /// string from pos.
+    constexpr basic_string_view substr(size_type pos, size_type n = npos) const noexcept
     {
         if (pos >= size())
             return basic_string_view();  // start past end -> return empty
@@ -247,7 +256,8 @@ public:
         return basic_string_view(data() + pos, n);
     }
 
-    OIIO_CONSTEXPR17 int compare (basic_string_view x) const noexcept {
+    /// Comparison function of two string_views, returning <0, 0, or >0.
+    constexpr int compare(basic_string_view x) const noexcept {
         // N.B. char_traits<char>::compare is constexpr for C++17
         const int cmp = traits_type::compare (m_chars, x.m_chars, (std::min)(m_len, x.m_len));
         return cmp != 0 ? cmp : int(m_len) - int(x.m_len);
@@ -305,11 +315,17 @@ public:
         return i == e ? npos : reverse_distance (this->crbegin(),i);
     }
 
-    size_type find_first_of (CharT c, size_t pos=0) const noexcept { return find (c, pos); }
+    /// Find the first occurrence of character `c` in the view, starting
+    /// at position `pos`.
+    size_type find_first_of(CharT c, size_t pos=0) const noexcept { return find (c, pos); }
 
-    size_type find_last_of (CharT c, size_t pos=npos) const noexcept { return rfind (c, pos); }
+    /// Find the first occurrence of character `c` in the view, starting
+    /// at position `pos`.
+    size_type find_last_of(CharT c, size_t pos=npos) const noexcept { return rfind (c, pos); }
 
-    size_type find_first_of (basic_string_view s, size_t pos=0) const noexcept {
+    /// Find the first occurrence of any character contained in string `s` in
+    /// the view, starting at position `pos`.
+    size_type find_first_of(basic_string_view s, size_t pos=0) const noexcept {
         if (pos >= size())
             return npos;
         const_iterator i = std::find_first_of (this->cbegin()+pos, this->cend(),
@@ -317,7 +333,9 @@ public:
         return i == this->cend() ? npos : std::distance (this->cbegin(), i);
     }
 
-    size_type find_last_of (basic_string_view s, size_t pos=npos) const noexcept {
+    /// Find the last occurrence of any character contained in string `s` in
+    /// the view, starting at position `pos`.
+    size_type find_last_of(basic_string_view s, size_t pos=npos) const noexcept {
         if (pos > size())
             pos = size();
         size_t off = size()-pos;
@@ -326,14 +344,18 @@ public:
         return i == this->crend() ? npos : reverse_distance (this->crbegin(), i);
     }
 
-    size_type find_first_not_of (basic_string_view s, size_t pos=0) const noexcept {
+    /// Find the first occurrence of any character not contained in string `s`
+    /// in the view, starting at position `pos`.
+    size_type find_first_not_of(basic_string_view s, size_t pos=0) const noexcept {
         if (pos >= size())
             return npos;
         const_iterator i = find_not_of (this->cbegin()+pos, this->cend(), s);
         return i == this->cend() ? npos : std::distance (this->cbegin(), i);
     }
 
-    size_type find_first_not_of (CharT c, size_t pos=0) const noexcept {
+    /// Find the first occurrence of a character other than `c` in the view,
+    /// starting at position `pos`.
+    size_type find_first_not_of(CharT c, size_t pos=0) const noexcept {
         if (pos >= size())
             return npos;
         for (const_iterator i = this->cbegin()+pos; i != this->cend(); ++i)
@@ -342,7 +364,9 @@ public:
         return npos;
     }
 
-    size_type find_last_not_of (basic_string_view s, size_t pos=npos) const noexcept {
+    /// Find the last occurrence of any character not contained in string `s`
+    /// in the view, starting at position `pos`.
+    size_type find_last_not_of(basic_string_view s, size_t pos=npos) const noexcept {
         if (pos > size())
             pos = size();
         size_t off = size()-pos;
@@ -350,7 +374,9 @@ public:
         return i == this->crend() ? npos : reverse_distance (this->crbegin(), i);
     }
 
-    size_type find_last_not_of (CharT c, size_t pos=npos) const noexcept {
+    /// Find the last occurrence of a character other than `c` in the view,
+    /// starting at position `pos`.
+    size_type find_last_not_of(CharT c, size_t pos=npos) const noexcept {
         if (pos > size())
             pos = size();
         size_t off = size()-pos;
@@ -360,42 +386,51 @@ public:
         return npos;
     }
 
-    friend OIIO_CONSTEXPR17 bool
+    /// Do two string views have the same sequence of characters?
+    friend constexpr bool
     operator==(basic_string_view x, basic_string_view y) noexcept
     {
         return x.size() == y.size() ? (x.compare(y) == 0) : false;
     }
 
-    friend OIIO_CONSTEXPR17 bool
+    /// Do two string views have the different sequences of characters?
+    friend constexpr bool
     operator!=(basic_string_view x, basic_string_view y) noexcept
     {
         return x.size() == y.size() ? (x.compare(y) != 0) : true;
     }
 
-    friend OIIO_CONSTEXPR17 bool
+    /// Is the first string view lexicographically less than the second?
+    friend constexpr bool
     operator<(basic_string_view x, basic_string_view y) noexcept
     {
         return x.compare(y) < 0;
     }
 
-    friend OIIO_CONSTEXPR17 bool
+    /// Is the first string view lexicographically greater than the second?
+    friend constexpr bool
     operator>(basic_string_view x, basic_string_view y) noexcept
     {
         return x.compare(y) > 0;
     }
 
-    friend OIIO_CONSTEXPR17 bool
+    /// Is the first string view lexicographically less than or equal to the
+    /// second?
+    friend constexpr bool
     operator<=(basic_string_view x, basic_string_view y) noexcept
     {
         return x.compare(y) <= 0;
     }
 
-    friend OIIO_CONSTEXPR17 bool
+    /// Is the first string view lexicographically greater than or equal to
+    /// the second?
+    friend constexpr bool
     operator>=(basic_string_view x, basic_string_view y) noexcept
     {
         return x.compare(y) >= 0;
     }
 
+    /// Stream output of a string_view.
     friend std::basic_ostream<CharT, Traits>&
     operator<<(std::basic_ostream<CharT, Traits>& out,
                const basic_string_view& str)
@@ -409,7 +444,7 @@ private:
     const CharT* m_chars = nullptr;
     size_t m_len = 0;
 
-    OIIO_CONSTEXPR14 void init(const CharT* chars, size_t len) noexcept
+    constexpr void init(const CharT* chars, size_t len) noexcept
     {
         m_chars = chars;
         m_len   = len;
@@ -430,20 +465,6 @@ private:
         return last;
     }
 
-    // Guaranteed constexpr length of a C string
-    static constexpr size_t cestrlen(const charT* chars) {
-#if OIIO_CPLUSPLUS_VERSION >= 17
-        return Traits::length(chars);
-#else
-        if (chars == nullptr)
-            return 0;
-        size_t len = 0;
-        while (chars[len] != 0)
-            len++;
-        return len;
-#endif
-    }
-
     class traits_eq {
     public:
         constexpr traits_eq (CharT ch) noexcept : ch(ch) {}
@@ -461,11 +482,6 @@ using wstring_view = basic_string_view<wchar_t>;
 
 
 
-// DEPRECATED name equivalence
-OIIO_DEPRECATED("Use string_view (2.3)")
-typedef string_view string_ref;
-
-
 /// Return a safe pointer to a null-terminated C string with the contents of
 /// the string_view.
 ///
@@ -490,6 +506,7 @@ typedef string_view string_ref;
 OIIO_UTIL_API const char* c_str(string_view str);
 
 
+// DEPRECATED(3.0)
 template<> inline const char*
 basic_string_view<char>::c_str() const {
     return OIIO::c_str(*this);
@@ -515,13 +532,6 @@ FMT_END_NAMESPACE
 // Declare std::size and std::ssize for our string_view.
 namespace std {
 
-#if OIIO_CPLUSPLUS_VERSION < 17
-template<class CharT, class Traits = std::char_traits<CharT>>
-constexpr size_t size(const OIIO::basic_string_view<CharT, Traits>& c) {
-    return c.size();
-}
-#endif
-
 #if OIIO_CPLUSPLUS_VERSION < 20
 template<class CharT, class Traits = std::char_traits<CharT>>
 constexpr ptrdiff_t ssize(const OIIO::basic_string_view<CharT, Traits>& c) {
